%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   This code is modified from the downloaded code of original guided filter paper. 
%       http://research.microsoft.com/en-us/um/people/kahe/eccv10/
%       "Guided Image Filtering", by Kaiming He, Jian Sun, and Xiaoou Tang, in TPAMI 2013.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [q] = guidedfilter(I, p, M, h, v, eps)

% image size
[hei, wid] = size(I); 
% the number of the sammpled pixels in each local patch
N = boxfilter_hv(M, h, v);
N(find(N == 0)) = 1;
% the size of each local patch; N=(2h+1)*(2v+1) except for boundary pixels.
N2 = boxfilter_hv(ones(hei, wid), h, v);

mean_I = boxfilter_hv(I.*M, h, v) ./ N;
mean_p = boxfilter_hv(p.*M, h, v) ./ N;
mean_Ip = boxfilter_hv(I.*p.*M, h, v) ./ N;

cov_Ip = mean_Ip - mean_I .* mean_p;
mean_II = boxfilter_hv(I.*I.*M, h, v) ./ N;
var_I = mean_II - mean_I .* mean_I;

% linear coefficients
a = cov_Ip ./ (var_I + eps);
b = mean_p - a .* mean_I;

mean_a = boxfilter_hv(a, h, v) ./ N2;
mean_b = boxfilter_hv(b, h, v) ./ N2;

% final output
q = mean_a .* I + mean_b;

end
