%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   This code is modified from the downloaded code of original guided filter paper. 
%       http://research.microsoft.com/en-us/um/people/kahe/eccv10/
%       "Guided Image Filtering", by Kaiming He, Jian Sun, and Xiaoou Tang, in TPAMI 2013.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [q] = guidedfilter_MLRI(G, R, mask, I, p, M, h, v, eps)

% image size
[hei, wid] = size(I); 
% the number of the sammpled pixels in each local patch
N = boxfilter_hv(M, h, v);
N(find(N == 0)) = 1;
% the size of each local patch; N=(2r+1)^2 except for boundary pixels.
N2 = boxfilter_hv(ones(hei, wid), h, v);

mean_Ip = boxfilter_hv(I.*p.*M, h, v) ./ N;
mean_II = boxfilter_hv(I.*I.*M, h, v) ./ N;

% linear coefficients
a = mean_Ip ./ (mean_II + eps);
N3 = boxfilter_hv(mask, h, v);
N3(find(N3 == 0)) = 1;
mean_G = boxfilter_hv(G.*mask, h, v) ./ N3;
mean_R = boxfilter_hv(R.*mask, h, v) ./ N3;
b = mean_R - a .* mean_G;

mean_a = boxfilter_hv(a, h, v) ./ N2;
mean_b = boxfilter_hv(b, h, v) ./ N2;

% final output
q = mean_a .* G + mean_b;

end
